import math
from typing import List, Dict

class BM25:
    def __init__(self, corpus: List[List[str]], k1: float = 1.5, b: float = 0.75) -> None:
        self.corpus = corpus
        self.corpus_size = len(corpus)
        self.avgdl = sum(len(doc) for doc in corpus) / self.corpus_size
        self.k1 = k1
        self.b = b

        self.doc_freqs: List[Dict[str, int]] = []
        self.idf: Dict[str, float] = {}

        df: Dict[str, int] = {}
        for document in self.corpus:
            frequencies: Dict[str, int] = {}
            for word in document:
                frequencies[word] = frequencies.get(word, 0) + 1
                if word not in df:
                    df[word] = 0
                df[word] += 1
            self.doc_freqs.append(frequencies)

        for word, freq in df.items():
            self.idf[word] = math.log(1 + (self.corpus_size - freq + 0.5) / (freq + 0.5))

    def calculate_bm25(self, document_index: int, query: List[str]) -> float:
        score = 0.0
        document = self.corpus[document_index]
        doc_len = len(document)
        for term in query:
            if term in self.doc_freqs[document_index]:
                term_freq = self.doc_freqs[document_index][term]
                sc = self.idf[term] * term_freq * (self.k1 + 1) / (term_freq + self.k1 * (1 - self.b + self.b * doc_len / self.avgdl))
                score += sc
        return score

    def rank(self, query: List[str]) -> List[float]:
        scores = [self.calculate_bm25(idx, query) for idx in range(self.corpus_size)]
        return scores
